
# q-expansion of newform 3840.2.a.bb, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3840
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [1], [2], [2], [2], [4], [4], [4], [2], [-4], [-2], [-6], [-4], [-8], [-10], [-6], [0], [12], [8], [6], [-4], [16], [6], [-14], [2], [-14], [8], [-16], [16], [-14], [6], [0], [16], [-18], [24], [10], [20], [-12], [-6], [10], [8], [-8], [-18], [-6], [-8], [-8], [2], [4], [-4], [-4], [0], [-26], [30], [8], [-28], [18], [24], [22], [-18], [-4], [-6], [-12], [32], [-2], [34], [20], [-10], [12], [-16], [4], [-24], [-6], [-14], [8], [32], [-2], [-10], [14], [-38], [-34], [-20], [24], [-34], [0], [-12], [14], [2], [6], [38], [-24], [-32], [-26], [6], [4], [0], [-6], [30], [-20], [-8], [-36], [2], [12], [38], [20], [-6], [8], [24], [0], [38], [-34], [-42], [4], [-8], [44], [18], [12], [-24], [-34], [-18], [-32], [-34], [22], [-40], [-20], [34], [52], [0], [-26], [46], [-4], [-32], [-28], [6], [14], [34], [10], [12], [-38], [34], [-40], [18], [38], [-28], [-16], [-24], [46], [-20], [-56], [-44], [2], [-2], [-28], [36], [-28], [48], [36], [-34], [2], [38], [12], [24], [-54], [-26], [-36], [-8], [16], [-62], [54], [-30], [2], [36], [48], [22], [-4], [-2], [-8], [38], [-2], [-20], [-10], [22], [-42], [24], [0], [-2], [-6], [44], [-58], [0], [-58], [4], [16], [-66], [0], [-24], [2], [2], [-48], [40], [30], [44], [2], [-6], [18], [-30], [-20], [-4], [-2], [4], [-34], [54], [-6], [-12], [0], [6], [2], [-18], [-48], [-62], [-68], [0], [-14], [14], [-16], [68], [48], [56], [-2], [10], [58], [56], [56], [-42], [44], [4], [50], [-10], [2], [-48], [-76], [-36], [30], [-16], [-4], [-72], [-14], [66], [52], [-20], [54], [34], [-60], [54], [-66], [62], [0], [4], [54], [14], [10], [-12], [-60], [70], [-48], [-52], [30], [-54], [68], [-26], [20], [4], [46], [-12], [38], [42], [12], [-8], [58], [30], [-12], [-16], [0], [-12], [-12], [80], [26], [-42], [-72], [66], [-10], [40], [-52], [-6], [54], [-2], [28], [-66], [42], [52], [-26], [30], [40], [-72], [-20], [26], [40], [-76], [24], [50], [-12], [2], [54], [-4], [-40], [34], [66], [64], [-2], [-22], [60], [62], [66], [-26], [-52], [-10], [-72], [36], [80], [-30], [-84], [-30], [76], [56], [48], [72], [-20], [-84], [38], [2], [26], [-36], [-38], [-16], [30], [-54], [-12], [-76], [-40], [62], [64], [-14], [74], [-78], [0], [12], [-40], [-54], [32], [36], [82], [-46], [48], [-78], [92], [82], [30], [26], [10], [42], [-40], [-64], [-54], [32], [-34], [58], [-48], [-14], [14], [66], [38], [-32], [-62], [12], [16], [60], [-12], [46], [20], [12], [34], [-70], [42], [-68], [-48], [-54], [-56], [-86], [-92], [14], [4], [72], [-14], [-96], [-14], [44], [-82], [46], [-36], [-34], [-62], [-42], [32], [60], [-82], [74], [24], [82], [72], [-68], [94], [-22], [8], [-58], [14], [18], [-64], [-6], [-52], [40]]
